#!/bin/bash

PROGRAM=daft.sh
VERSION=0.6
VERSTAG=devel-20150402-14-TAW
AUTHOR="Tsjerk A. Wassenaar, PhD"
YEAR="2015"
AFFILIATION="University of Erlangen/Nuernberg"

DESCRIPTION=$(cat << __DESCRIPTION__
__DESCRIPTION__
)


#   I. TM[]       -> PDB[]        ; Convert sequences to helix PDB
#  II. PDB[]      -> (CG, ITP)[]  ; Martinize each PDB
# III. (CG,ITP)[] -> (SYS,TOP)[N] ; Build a system from the different parts, repeat N times


# Get script directory
#
# A: Test if program name contains a slash        (1) 
#    If so, change directory to program directory (2)
# B: Echo working directory, which will always be 
#    the program directory
#baseName
#       |---------------A---------------|  |B|
#       |--------1--------|    |---2----|
SDIR=$( [[ $0 != ${0%/*} ]] && cd ${0%/*}; pwd )


# - main options:
AA=()
CG=()
TM=()
COMBI=()
NUM=1
NAME=daft
FF=martini22
TIME=500
SOL=W
LOW=()
UPP=()

PROC=0
DIST=3
DZ=2

D3D=
ADS=
MEM=5.0

# - martinizing - default settings
MARTINIZEOPTS=()

# - insane - default settings
INSANE0=(-ring -pbc=keep)
INSANEOPTS=()

# - martinating - default settings
MARTINATE=(-salt=-0 --mdp-gen_vel=yes)

# Pymol executable
PYMOL=pymol

# Gromacs RC file
GMXRC=

OPTIONS=$(cat << __OPTIONS__

## OPTIONS ##

  DAFT options:

    -aa          Atomistic input file (M):                                    *FILE:  None

                     [NAME=]file.pdb

                 $PROGRAM will pass these files to martinize.py for 
                 conversion to coarse-grained. The object can be 
                 named using the syntax NAME=AA.pdb. This allows 
                 referencing for building multiple assays simultaneously.

    -cg          Coarse grained input file (M):                               *FILE:  None

                     [NAME=]file.pdb,file.top

                 Coarse grained input must consist of a structure AND 
                 topology. The object can be named by using the syntax 
                 NAME=CG.pdb,CG.top. The name allows referencing for 
                 building multiple assays simultaneously.

    -tm          Transmembrane peptide sequence (M)                           *STR:   None

                     [NAME=]sequence[,secstruc]

                 By default the sequence is built as helix, but it is 
                 possible to specify the secondary structure. The 
                 structure is built using PyMOL, and then converted to
                 coarse-grained using martinize.py

    -c           Combination of objects (M)                                   *STR:   None

                     NAME[,NAME[,NAME[...]]]

                 Different combinations of named objects can be specified,
                 each of which will be used to set up an assay with the 
                 specified parameters. If only a number is given, then
                 all possible combinations of identical units will be set 
                 up for that number of components. If an exclamation mark
                 is added, all combinations of non-identical units will 
                 be set up. Thus -c 2 -c 2! gives all pairs. 

    -n           Number of runs                                               *INT:   $NUM

    -name        Name of run                                                  *STR:   None

    -p           Number of processes to run simultaneously                    *STR:   0

    -pymol       PyMOL executable                                             *FILE:  $(which pymol)

    -dssp        DSSP executable                                              *FILE:  $(which dssp)

    -3d          Set up 3D assay                                              *BOOL:  FALSE
 
    -ads         Set up adsorption assay with distance specified              *FLOAT: None

    -mem         Reference membrane thickness for adsorption assay            *FLOAT: $MEM

    -ff          Force field to use                                           *STR:   $FF

    -sol         Solvent to use                                               *STR:   $SOL

  Passing options to auxiliary programs:

    --insane-opt=val    Option to pass to insane
    --martinize-opt=val Option to pass to martinize
    --martinate-opt=val Option to pass to martinate

                 Every -opt=val is added to the command line of the 
                 call to the corresponding program as -opt val
                 To specify multiple options in one go the following
                 syntax can be used:

                 --program{-opt1=val,-opt2=a\,b\,c,-opt3=}

__OPTIONS__
)


USAGE ()
{
    cat << __USAGE__

$PROGRAM version $VERSION:

$DESCRIPTION

$OPTIONS

(c)$YEAR $AUTHOR
$AFFILIATION

__USAGE__
}


BAD_OPTION ()
{
  echo
  echo "Unknown option "$1" found on command-line"
  echo "It may be a good idea to read the usage:"
  echo -e $USAGE

  exit 1
}


# Function for handling argument lists for downstream programs
# 1. Expanding options from '-opt=val1\,val2' to '-opt val1 val2', not changing long options (--long-opt=val)
function expandOptList()   { for i in $@; do [[ $i =~ --+ ]] && echo $i || (j=${i/=/ }; echo ${j//\\,/ }); done; }
# 2. Condensing options from '-opt1=val1,val2 -opt2=val3,val4' to '-opt1=val1\,val2,-opt2=val3\,val4' 
function condenseOptList() { echo $(sed 's/,/\,/g;s/  */,/g;' <<< $@); }
# 3. Reading condensed options from the command line
function readOptList() { sed "s/\\\,/##/g;s/,/ /g;s/##/,/g;s/--[^{]\+{\(.*\)}/\1/;" <<< $1; }


# Counter for items
ID=1

while [ -n "$1" ]; do
  case $1 in
              -h)                    USAGE       ; exit 0 ;;

    # File options
             -aa)            AA+=($((ID++))=$2)  ; shift 2; continue;;
             -cg)            CG+=($((ID++))=$2)  ; shift 2; continue;;
             -tm)            TM+=($((ID++))=$2)  ; shift 2; continue;;
          -pymol)                 PYMOL=$2       ; shift 2; continue;;
          -gmxrc)                 GMXRC=$2       ; shift 2; continue;;
           -dssp)                  DSSP=$2       ; shift 2; continue;;
    # Control options
             -ff)                    FF=$2       ; shift 2; continue;;
           -time)                  TIME=$2       ; shift 2; continue;;
              -d)                  DIST=$2       ; shift 2; continue;;
             -dz)                    DZ=$2       ; shift 2; continue;;
            -sol)                   SOL=$2       ; shift 2; continue;;
              -c)                COMBI+=($2)     ; shift 2; continue;;
              -n)                   NUM=$2       ; shift 2; continue;;
              -p)                  PROC=$2       ; shift 2; continue;;
           -name)                  NAME=$2       ; shift 2; continue;; 
             -3d)                   D3D=-3d      ; shift  ; continue;; 
            -ads)                   ADS="-ads $2"; shift 2; continue;; 
            -mem)                   MEM=$2       ; shift 2; continue;; 

    # Options for downstream programs
    # If the options are given on the command line, they are expanded and each
    # option will be formatted as --program-opt=val
    --martinate-*) MARTINATE+=(${1#--martinate}) ; shift  ; continue;;
    --martinize-*) MARTINIZEOPTS+=(${1#--martinize}) ; shift  ; continue;;
    --insane-*)       INSANEOPTS+=(${1#--insane})    ; shift  ; continue;;

    # If the options are passed by another program, they will be formatted like
    #   --program{-opt1=val1,-opt2=val2\,val3}
    # In this case the option needs to be parsed explicitly:
    --martinate*)  MARTINATE+=($(readOptList $1)) ; shift  ; continue;;
    --martinize*)  MARTINIZEOPTS+=($(readOptList $1)) ; shift  ; continue;;
    --insane*)        INSANEOPTS+=($(readOptList $1)) ; shift  ; continue;;

    # All options should be covered above. Anything else raises an error here.
         *)         BAD_OPTION $1;;
  esac
done


cat << __RUNINFO__

$PROGRAM version $VERSION:

(c)$YEAR $AUTHOR
$AFFILIATION

Now executing...

$CMD

__RUNINFO__


# Macros to echo stuff only if the step is to be executed
SHOUT() { echo && sed -e 's/^/ /;s/$/ /;h;s/./-/g;p;x;p;x;' <<< "$@" | sed 's/^/#/'; }
LINE()  { echo && sed -e 's/^/ /;s/$/ /p;s/./-/g;' <<< "$@" | sed 's/^/#/'; }

# Macro to do stuff only if the step is to be executed
DO() { echo "$@" && $@; }

# Sequence function that also works on Mac
SEQ(){ for ((__i=$1; __i<=$2; __i++)); do echo $__i; done };


#---------------------------------------------------------------------
## DAFT-0: NO COMPONENTS, ONLY SETTING MARTINATE
#---------------------------------------------------------------------


if [[ -z $TM && -z $AA && -z $CG ]]
then

    #---------------------------------------------------------------------
    SHOUT "---= DAFT 0: RUNNING LIPIDOMICS ASSAY =---"
    #---------------------------------------------------------------------

    INSANE="--insane{$(condenseOptList ${INSANEOPTS[@]}),-sol=$SOL,-d=$DIST,-dz=$DZ}"
    MARTINATE="\$MART $(expandOptList ${MARTINATE[@]}) $INSANE"
    MARTINATE="$MARTINATE -time $TIME -cg $FF -sol $SOL ${GMXRC:+-gmxrc $GMXRC}"
    
    [[ -d $NAME ]] || mkdir $NAME
    cd $NAME
 
    # Wind up to the first free directory
    N=1; while [[ -d $NAME-$(printf "%04d" $N) ]]; do : $((N++)); done
    M=$((N+NUM))
    for ((; N<$M; N++))
    do
	DIR=$NAME-$(printf "%04d" $N)
	mkdir $DIR
	{ echo '#!/bin/bash'; echo "MARTINATE=\${1:-$SDIR/martinate.sh}"; echo $MARTINATE; } > $DIR/martinate-run.sh
    done

    exit
fi


#---------------------------------------------------------------------
## BOOKKEEPING                                          
#---------------------------------------------------------------------


# 1. Array for temporary files
GARBAGE=()


# 2. Command lines for downstream programs
#    (martinize, insane, martinate)

MARTINIZE="$SDIR/martinize.py $(expandOptList ${MARTINIZEOPTS[@]}) -ff $FF"

# Cannot use this with new cmdline syntax. Need to set -excl -1 by default if
# no membrane is requested with insane.
#[[ -z $SOL ]] && SOL="-sol W"   || SOL=$(for i in ${SOL[@]}; do echo -sol $i; done)
[[ " ${INSANEOPTS[@]}" =~ " -u=" ]] && MEMBRANE="--insane-l=TRUE"
[[ " ${INSANEOPTS[@]}" =~ " -l=" ]] && MEMBRANE="--insane-l=TRUE"
INSANE="$SDIR/insane.py $(expandOptList ${INSANE0[@]}) $(expandOptList ${INSANEOPTS[@]}) -sol $SOL"

MARTINATE="$(expandOptList ${MARTINATE[@]})"
MARTINATE="$MARTINATE -time $TIME -cg $FF -sol $SOL ${GMXRC:+-gmxrc $GMXRC} $MEMBRANE"


# 3. Sed statement for converting item names to IDs
#    Updated later on for TM and AA groups
TAGS2NRS=""
for i in ${CG[@]}
do 
    id=${i%%=*}
    j=${i#*=}
    [[ $j =~ = ]] && TAGS2NRS="${TAGS2NRS}s/\([ ,]*\)${j%%=*}\([ ,]*\)/\1@$id\2/g;"
done


#---------------------------------------------------------------------
SHOUT "---= STEP 1: HELIX GENERATION FROM COMMAND LINE SEQUENCES =---"
#---------------------------------------------------------------------

counter=1

for tm in ${TM[@]}
do 
    SHOUT "Building TM peptide $((counter++))"
    echo 
    echo $tm

    # There's always an ID
    ID=${tm%%=*}
    tm=${tm#*=}

    # Set the name for this item
    [[ $tm =~ = ]] && ITEM=${tm%%=*} || ITEM=_TM$ID

    # Remove the name from the argument
    tm=${tm#*=}

    # Check whether the helix needs to be put upside down
    [[ ${tm:0:1} == "-" ]] && INV=-inv || INV=
    
    # Remove the dash
    tm=${tm#-}

    # Set the secondary structure
    # This is set for martinize.
    [[ $tm =~ , ]] && secstruc=${tm##*,} || secstruc=H

    # Numbers can be used to repeat types
    # L3H6L3 -> LLLHHHHHHLLL
    secstruc=$(sed 's/[[:alpha:]]/ &/g;s/[[:digit:]]\+/& /g' <<< $secstruc)
    secstruc=$(for i in $secstruc; do k=${i:0:1}; echo $k; for ((j=0; j<${i:1}-1; j++)); do echo $k; done; done)
    secstruc=$(sed 's/ //g' <<< $secstruc)

    # Remove the sequence from the argument
    tm=${tm%,*}
    tm=${tm%:*}

    # Give some info
    printf "\nItem:      %s\nSequence:  %s\nStructure: %s\n\n\n" $ITEM $tm $secstruc

    # Set secondary structure for PyMOL, 
    # based on whatever occurs most
    H=$(wc -c <<< ${secstruc//[^H]/})
    S=$(wc -c <<< ${secstruc//[^S]/})
    L=$(wc -c <<< ${secstruc//[^L]/})
    s=1
    [[ $L -gt $H ]] && [[ $L -gt $S ]] && s=0
    [[ $S -gt $H ]] && [[ $S -gt $L ]] && s=2

    itemFile=$ITEM.pdb

    [[ -n $ADS ]] && turn=1 || turn=0

    cat << __PML__ > $ITEM.pml
set secondary_structure, $s
editor.build_peptide('$tm')
res0=int(cmd.get_model().atom[0].resi)
create tmp, all
alter_state 1, tmp , (x,y,z)=(0,0,${INV}(int(resi)-res0-0.5*${#tm}))
align not tmp, tmp
delete tmp 
remove hydro
mx,my,mz=zip(*cmd.get_model().get_coord_list())
mx,my,mz=sum(mx)/len(mx),sum(my)/len(my),sum(mz)/len(mz)
if not $turn: cmd.alter_state(1, "all", "(x,y,z)=(x-mx,y-my,z-mz)")
if $turn:     cmd.alter_state(1, "all", "(x,y,z)=(z-mz,y-my,mx-x)")
save $itemFile
__PML__

    # ## Set the Pymol command line
    # # Set secondary structure
    # PYMOLCMD="cmd.set('secondary_structure','$s')"
    # # Build the peptide
    # PYMOLCMD="$PYMOLCMD;editor.build_peptide('$tm')"
    # # Neat trick to align the helix with the z-axis (following three lines of code)
    # # - First copy the peptide
    # PYMOLCMD="$PYMOLCMD;res0=int(cmd.get_model().atom[0].resi);cmd.create('tmp','all')"
    # # - Set the coordinates of the copy to lie along the z-axis (mind inversion)
    # PYMOLCMD="$PYMOLCMD;cmd.alter_state(1,'tmp','(x,y,z)=(0,0,${INV}(int(resi)-res0-0.5*${#tm}))')"
    # # - Align the helix with the copy; The hack with cmd.get_object_list is requireed
    # #   because the command line option of Pymol does not allow spaces :S
    # PYMOLCMD="$PYMOLCMD;cmd.align(cmd.get_object_list()[0],'tmp')"
    # # Now shift to make sure the center is at (0,0,0)
	
    # # If this is an adsorption study and we do not rotate in 3D, then rotate by 90 degrees over y

    # # Write out
    # PYMOLCMD="$PYMOLCMD;cmd.delete('tmp');cmd.remove('hydro');sys.stdout.write(cmd.get_pdbstr())"

    # # Display the pymol script executed
    # LINE "PyMOL script:"
    # sed 's/;/\n/g' <<< $PYMOLCMD
    # echo    

    # Create the helix structure in pymol
    # Remove hydrogen atoms
    # Align the helix with the z-axis

    [[ -f $itemFile ]] && echo Output found. Skipping structure building... || $PYMOL -qc $ITEM.pml || exit
	
    # Convert structure to coarse grained and build topology using martinize.py
    out=${itemFile%.pdb}
    MART="${MARTINIZE[@]} -f $itemFile -name $ITEM -o $ITEM.top -x $ITEM-mart.pdb -n $ITEM-mart.ndx -ss $secstruc"
    echo $MART
    [[ -f $ITEM-mart.pdb ]] && echo Output found. Skipping martinizing... || $MART || exit

    # If we use polarizable or BMW water, we need to change AC1/AC2 to C1/C2
    #$POLARIZABLE && sed -i -e 's/AC[12]/ C[12]/' $ITEM-mart.pdb

    # Add helix to coordinate file list 
    CG+=($ID=$ITEM=$ITEM-mart.pdb,$ITEM.top)

    # Add replacement statement
    TAGS2NRS="${TAGS2NRS}s/\([ ,]*\)$ITEM\([ ,]*\)/\1@$ID\2/g;"
    
    # Increment the ID and CHAIN counters
    : $((CHAIN++))
	
done


#-------------------------------------------------------
SHOUT "---= STEP 2: MARTINIZE ATOMISTIC STRUCTURES =---"
#-------------------------------------------------------

counter=1

for aa in ${AA[@]}
do

    SHOUT "Martinizing AA structure $((counter++))"
    echo $aa

    # There's always an ID
    ID=${aa%%=*}
    aa=${aa#*=}

    # Set the name for this item
    [[ $aa =~ = ]] && ITEM=${aa%%=*} || ITEM=_AA$ID

    # Remove the name from the argument
    aa=${aa#*=}

    # Set the secondary structure if given, point to dssp otherwise
    [[ $aa =~ , ]] && secstruc="-ss ${aa##*,}" || secstruc="-dssp $DSSP"

    # Remove the sequence from the argument
    aa=${aa%,*}

    # Give some info
    printf "\nItem:      %s\nSequence:  %s\nStructure: %s\n\n\n" $ITEM $aa "$secstruc"

    # Convert structure to coarse grained and build topology using martinize.py
    MART="${MARTINIZE[@]} -f $aa -name $ITEM -o $ITEM.top -x $ITEM-mart.pdb -n $ITEM-mart.ndx $secstruc"
    echo $MART
    [[ -f $ITEM-mart.pdb ]] && echo Output found. Skipping martinizing... || $MART || exit

    # If we use polarizable or BMW water, we need to change AC1/AC2 to C1/C2
    #$POLARIZABLE && sed -i -e 's/AC[12]/ C[12]/' $ITEM-mart.pdb

    CG+=($ID=$ITEM=$ITEM-mart.pdb,$ITEM.top)                

    TAGS2NRS="${TAGS2NRS}s/\([ ,]*\)$ITEM\([ ,]*\)/\1@$ID\2/g;"
done


#----------------------------------------------------------
SHOUT "---= STEP 3: GENERATE DAFT STARTING STRUCTURES =---"
#----------------------------------------------------------

# Strip one tag (*=) from the start of a string
STRIP() { sed 's/[^=]*=//' $@; }

# Sort the items, so they are passed to daft.py in the original order
ITEMS=($(for i in ${CG[@]}; do echo $i; done | sort -n | sed 's/[^=]*=//' ))

# List the items we have
echo
echo "Items in order given:"
for ((i=0; i<${#ITEMS[@]}; i++))
do
    tag=${ITEMS[$i]%=*}
    item=${ITEMS[$i]#*=}
    [[ $tag == ${ITEMS[$i]} ]] && tag='[NONE]'
    printf "%4d %6s = %s (%s)\n" $((i+1)) $tag ${item%%,*} ${item#*,} 1>&2
done
echo

# Replace item names by numbers
COMBI=($(sed "$TAGS2NRS" <<< " ${COMBI[@]} "))

# Get structures
STRUC=$(for i in ${ITEMS[@]}; do echo "-f ${i%%,*}"; done)

# Set combination options for daft.py
[[ -n $COMBI ]] && COMBI="-c $(sed 's/  */ -c /g' <<< ${COMBI[@]})" 

# Generate starting structures - each is put in a numbered directory
DAFT="$SDIR/daft.py -name $NAME -d $DIST -n $NUM -z $DZ $STRUC $COMBI $D3D $ADS -mem $MEM"
echo $DAFT
echo
STUFF=($( $DAFT | tee /dev/stderr | grep "^@ ")) 2>&1
echo

# Close stuff with a last '@'
STUFF+=(@)

#--------------------------------------------------
SHOUT "---= STEP 4: GENERATE INSANE MEMBRANES =---"
#--------------------------------------------------

# This is done per combination generated by daft.py
# For each combination we need to build a topology 
# file from the files generated by martinize. 
echo ${STUFF[@]} ${#STUFF[@]}

for ((i=0; i<${#STUFF[@]}-1; ))
do
    # Parse the output from daft that was stored in
    # the variable STUFF. This is formatted like:
    #
    # @ TM1-TM2/daft-%04d/daft.pdb  2 3 : 1 2
    #

    # Skip the '@'
    : $((i++))
    # File name template
    files=${STUFF[$((i++))]}
    # Start of sequence
    start=${STUFF[$((i++))]}
    # End of sequence (exclusive)
    end=${STUFF[$((i++))]}
    # Skip the ':'
    : $((i++))
    # Store the items
    # These correspond to the original IDs
    items=()
    while [[ ! ${STUFF[$i]} == "@" ]]; do items+=(${STUFF[$((i++))]}); done

    echo
    echo "Building systems for combination ${files%%/*}, $((end-start)) structures ($start to $end)."
    # Gather the #includes for the molecule types, and generate the topology
    # The prototopology filename is taken from the structure filename,but without the number
    TOP=${files%-%*}.top
    DIR=${TOP%/*}
    tops=($(for j in ${items[@]}; do echo ${ITEMS[$((j-1))]##*,}; done))
    (
        echo '#include "martini.itp"'
        echo '#include "martini_v2.0_lipids.itp"'
        echo '#include "martini_v2.0_ions.itp"'
	grep -h '#include' ${tops[@]} | grep -v '"martini\.itp"' | sort -u 
	echo
	echo "[ system ]"
	echo "DAFT setup for ${files%%/*}"
	echo
	echo "[ molecules ]"
        # '$a;' is to ensure the file ends with a newline 
	for j in ${tops[@]}; do sed -e '1,/\[ *molecules *\]/d' $j; echo ';'; done
    ) > $TOP

    # Call insane for every structure and copy the itp files
    for ((j=$start; j<$end; j++))
    do
        # Set the filename with number
	F=$(printf "$files" $j)
	G=${F%.pdb}.gro
        # Set the corresponding topology filename
	T=${F%.pdb}.top
	# Set the corresponding directory
	D=${F%/*}
	# Combine the prototopology with the molecule list given by insane
        #echo $INSANE -d $DIST -f $F -o $G
	{ cat $TOP; $INSANE -d $DIST -f $F -o $G 2>&1; } > $T
	# Copy the itp files needed
	cp $(sed -n '/martini.*\.itp/d;s/#include  *.\(.*\.itp\)./\1/p;' $TOP) ${F%/*}/
	# Copy the index file and add the groups generated by insane
	cp $DIR/$NAME.ndx $D/
	NDX=($(grep '; NDX' $T))
	for ((k=0; k<${#NDX[@]}; k+=5))
	do
	    echo "[ ${NDX[$((k+2))]} ]" >> $D/$NAME.ndx
	    SEQ ${NDX[$((k+3))]} ${NDX[$((k+4))]} >> $D/$NAME.ndx
	done

	# Write the run script
        { 
            echo '#!/bin/bash'
            echo "MARTINATE=\${1:-$SDIR/martinate.sh}"
            echo \$MARTINATE $MARTINATE -f ${G##*/} -top ${T##*/} -step EM -daft $NAME.ndx
        } > ${F%/*}/martinate-run.sh

	#echo $MARTINATE -f ${G##*/} -top ${T##*/} -step EM -daft $NAME.ndx > ${F%/*}/martinate-run.sh
	printf .	
    done
done
echo


exit

MART="$SDIR/martinate.sh -f $NAME.pdb ${M_ARGS[@]} -pbc retain -Lring -daft"
echo $MART

JOB=$(cat << __JOB__
#!/bin/bash

$MART -stop TPR
__JOB__
)

counter=0

if [[ $PROC -gt 0 ]]
then
    for i in $(ls | grep "$name\-[0-9]\{5,\}\$")
    do
        echo -e "$JOB" > $i/job.sh
    done

    # Run the simulations in parallel, according to the number of processes
    ls | grep "$name\-[0-9]\{5,\}\$" | xargs -P $PROC -n 1 -I@ bash -c "cd @; [[ \$(( \$(sed s/^.*-0*// <<< @) % 50 )) == 0 ]] && echo -n 0 || echo -n .; $MART -stop TPR >martinate.log 2>&1"
else
    # Prepare each directory such that it can be run on its own.
    # Since martinize.py depends on dssp and we cannot be certain about its availability,
    # the topologies have to be prepared before...
    for i in $(ls | grep "$name\-[0-9]\{5,\}\$")
    do 
	(cd $i; $MART -stop TOP >martinate.log 2>&1)
	echo -e '#!/bin/bash\n\n./martinate.sh -f '$NAME.pdb ${M_ARGS[@]}' -step TOP -gmxrc $1 -daft >daft-run.log' > $i/daft-run.sh
	cp $SDIR/*.sh $SDIR/*.py $i
	chmod +x $i/*.sh $i/*.py 
        : $((counter++))
        [[ $((counter%10)) == 0 ]] && echo -n $counter || echo -n .
    done
    echo
fi


